.modal-overlay {
  position: fixed;  
  top: 0;  
  right: 0;  
  bottom: 0;  
  left: 0;  
  height: 100%;  
  width: 100%;  
  margin: 0;  
  padding: 0;  
  background: #e3e3e3;
  opacity: .85;  
  filter: alpha(opacity=60);  
  z-index: 9500;  
}  
div.modal-window {  
  background: #ffffff;  
  z-index: 9501;  
  position: fixed;
  top: 0;
  width: 980px;
  height: 100%;
  overflow: hidden;
}
div.modal-window iframe {
  width: 100%;
  height: 100%;
  overflow: auto;
}
a#close-window {  
  position: absolute;  
  width: 77px;  
  height: 24px;  
  right: 14px;  
  top: 40px;  
  background: transparent url(close.png) no-repeat scroll right top;  
  text-indent: -99999px;  
  overflow: hidden;  
  cursor: pointer;  
}  
