
/*bootstrap layout additions--------------------------------------------------*/
body { font-family: Verdana, Arial, sans-serif; }
.footer {
  margin-top: 20px;
  padding: 35px 0 36px;
  border-top: 1px solid #e5e5e5;
}
.footer p { margin-bottom: 0; color: #555; }
.navbar .brand {
	background-image: url(imgs/logocms.png);
	background-repeat: no-repeat;
	background-position: left top;
	margin-top:8px;
	margin-right:12px;
	padding:0px;
	height:25px;
	width:85px;
}

/*form additions--------------------------------------------------*/
form label.required:after  { content: "*"; }
form input.password        { display:block; }

/*elements--------------------------------------------------*/
.pageelement { margin-top:6px; }
.peSeparator { clear:both; margin-bottom: 0px; }
.peSeparator.bo { height: 0px; margin-top:2px; margin-bottom:2px; border-top:1px solid #333333;}
.peSeparator.h0 { height: 3px; }
.peSeparator.h1 { height: 1em; }
.peSeparator.h2 { height: 2em; }
.peFile a {
	display: block;
	padding-left: 20px;
	background-image: url(imgs/download.png);
	background-repeat: no-repeat;
	background-position: left top;
}
.peImage div.title { color: #666666; font-size: 90%; }
.peImage div.left  { margin-right: 20px; margin-bottom: 10px; float:left; }
.peImage p { padding-bottom: 0px; margin-bottom: 0px; }
.peImage .teaser  { margin-bottom: 20px; }
.peImage .teaserl { margin-bottom: 20px; }
.peImage .teasers { margin-bottom: 20px; }

.peNews { clear:both; }
.peNews .header img { display: block; margin-right: 10px; float: left; margin-bottom: 10px; }
.peNews .teaser img { display: block; width: 220px; margin-right: 20px; float: left; margin-bottom: 10px; }
.peNews .teaser_s img { display: block; width: 100px; margin-right: 10px; float: left; margin-bottom: 10px; }
.peNews h3 { font-family: Verdana, Arial, sans; font-size: 100%; }
.peNews .teaser .itext { margin-left: 240px; }
.peNews .teaser_s .itext { margin-left: 110px; }
.peNews .newsline { margin-bottom:5px; }
.peNews .newsline h4 { width: 240px; float:left; padding:0px; margin:0px; }
.peNews span.date { color:#333333; }
.peNews span.dateright { float:right; margin-right:10px; color:#333333; }
.peNote { background-color:#eeff00; border: 1px solid #cecece; padding: 10px; margin-bottom: 12px; }


