body {
	font-family: Verdana, Arial, sans-serif;
	font-style: normal;
	padding: 0px;
	margin: 0px;
	font-size: 80%;
}

#header { 
  width: 100%; height: 30px; background-color:#aaaaaa; 
  background-image: url(images/logocms.png);
  background-repeat: no-repeat;
  background-position: 870px 3px;
}
#header h1 {
  font-family: courier, monospace;
  font-size:13pt;
  font-weight:bold;
  line-height: 1.2em;
  color: white;
  margin: 0px;
  padding: 4px 12px;
  width: auto;
}
#nive_edit { padding: 10px; min-height: 400px; }
#breadcrumbs { clear:both; padding: 10px 0px 0px 10px; color: #999999; }
#main { clear: both; }
#container {
	text-align: left;
	vertical-align: top;
	width: 960px;
	margin-left: auto;
	margin-right: auto;
}

h1     { font-size: 160%; line-height: 1.3em; }
h2     { font-size: 140%; line-height: 1.3em; }
h3     { font-size: 120%; line-height: 1.3em; }
h4     { font-size: 100%; line-height: 1.3em; }
.btn   { font-size: 100%; line-height: 1.3em; }
.block { width: 48%; margin: 1%; float:left; }

/*form styling --------------------------------------------------*/
form.form .control-label { width: 150px; }
form.form .controls      { width:520px; margin-left:20px; float:left; }
form.form .controls input, textarea, select { width:500px; }
form.form .controls input[type="radio"] { width:auto; margin-bottom: 2px; display:block; }
form.form .controls input[type="checkbox"] { width:auto; margin-bottom: 2px; display:block; }
form.form .help-block    { width:220px; margin-left:20px; display:block; float:left; }
form label.required:after  { content: "*"; }
form input.password        { display:block; }

/*tabs--------------------------------------------------*/
nav .spacer { width:10px; height:13px; float:left; border-right: 1px solid #ffffff; color: #aaaaaa; background-color: #aaaaaa; }
nav a.tab {
	display:block;
	padding-left: 8px;
	padding-right: 8px;
	padding-top: 4px;
	padding-bottom: 4px;
	height: 17px;
	float: left;
	color: #FFFFFF;
	text-decoration: none;
	font-size: 100%;
	font-weight: bold;
	background-color: #aaaaaa;
	border-right: 1px solid #ffffff;
	border-top: 1px solid #ffffff;
}
nav a.tab:hover { background-color: #999999; }
nav a.activeTrue { background-color: #686868; }
nav .btn-group {margin-top:5px;}


/* error handling, messages */
.alert-error { border: 1px solid #999999; background-color: #ffcaca; width: auto; padding: 10px; }
.alert-info { border: 1px solid #999999; background-color: #ecffc6; width: auto; padding: 0px; }
.alert ul { margin:0px; padding:0px; }
.alert li { margin:0px; padding:0px;list-style: none; }
.poolform .req { color: red; }
.poolform .error { background-color: #FFDFDF; }
.poolform .error label.desc { color: #DF0000; }
.poolform .errorMsg, .poolform .errorMsgLbl { font-size: 90%; padding:0px; margin:0px; }

a.button {
  cursor: pointer;
  font-weight: bold; 
  background-color: #686868;
  margin-top:10px;
  padding: 2px 20px 2px 20px;
  color: white;
  width: auto; 
  text-decoration:none;
}

/*sorting--------------------------------------------------*/
#sortable { list-style-type: none; padding: 0; width: auto; margin:0px; margin-bottom:30px;}
#sortable li { 
  margin: 0 3px 3px 3px; padding: 0.4em; padding-left: 1.5em; height: 16px; cursor: pointer; width: 50%; 
  border: 1px solid #cdcdcd; background-color: #F4F4F4;
}

/*addlist--------------------------------------------------*/
.addList {
  display: block;
}
.addList h4 {
  clear:both;
  display: block;
  padding:4px;
  margin:0px;
  padding-top:10px;
  border-bottom: 1px solid #cdcdcd;
}
.addList a {
  margin: 0px;
  display: block;
  padding: 5px;
  float:left;
  width:220px;
  color: black;
  text-decoration: none;
}
.addList a img {
  padding-right: 10px;
}
.addList a:hover {
  background-color: #eaeaff;
  text-decoration:none;
}

